#ifndef __CImageSegmentation_H_2005_03_23_15_44_W_AI__
#define __CImageSegmentation_H_2005_03_23_15_44_W_AI__ 1

#include <vector>
#include <map>
#include <assert.h>

namespace segmentation {

// image segmentation - manages a matrix of segment indexes for each pixel
template <class INDEX_TYPE = signed long int, INDEX_TYPE idxUndefinedSegment = -1>
class TImageSegmentation
{
public:
  typedef INDEX_TYPE THE_INDEX_TYPE;
  enum { IdxUndefinedSegment = idxUndefinedSegment};

  // create non initialized segmentation object
  TImageSegmentation(long width=0, long height=0);

  // create copy of supplied segmentation object
  // (not really needed for now 'cause all the members have copy constructor)
  TImageSegmentation(const TImageSegmentation & copy);

  // destructor :-)
  ~TImageSegmentation();

  // create matrix with specified dimensions and initialize it all with one segment (index 0)
  bool Init(long width, long height);

  // direct access to index buffer
  inline INDEX_TYPE * operator * ();
  inline const INDEX_TYPE * operator * () const;

  // access to value in the index buffer
  inline INDEX_TYPE & operator () (const long x, const long y);
  inline const INDEX_TYPE & operator () (const long x, const long y) const;

  // return width of index matrix
  INDEX_TYPE Width() const
  { return mWidth; }

  // return height of index matrix
  INDEX_TYPE Height() const
  { return mHeight; }

  // determines and returns the used indexes (returns false if object not initialized)
  // (the size of each segment is the value of each element in the map)
  bool GetUsedSegmentIndexes(std::map<INDEX_TYPE, size_t> &indexSet) const;

  // reorders the segment indexes consecutively starting with 0
  bool ReorderSegmentIndexes();

  // returns the number of segments
  long GetSegmentCount() const;


protected:
  typedef std::vector<INDEX_TYPE> INDEX_ARRAY;
  
  INDEX_ARRAY mIndexArray;
  long mWidth;
  long mHeight;


  // asserts if objects state is not valid and returns true if object is initialized
  inline
  bool InitDone() const
  {
    AssertValid();
    return mWidth>0;
  }

#if defined(DEBUG) || defined(_DEBUG)

  inline
  void AssertValid() const // assert if object state is not valid
  {
    assert((mWidth==0 && mHeight==0 && mIndexArray.size()==0) || 
      (mWidth>0 && mHeight>0 && mIndexArray.size()==mWidth*mHeight));
  }

  inline
  void AssertNonEmpty() const // assert if object state is not valid or uninitialized
  {
    assert(InitDone()); // this should be enough because the whole consisteny is tested in AssertValid
  }

#else // Release versions of these functions are empty
  inline
  void AssertValid() const {}

  inline
  void AssertNonEmpty() const {}
#endif

};

} // namespace segmentation

#include "TImageSegmentation.txx"

#endif

