#ifndef __CImageSegmentation_TXX_2005_03_23_16_33_W_AI__
#define __CImageSegmentation_TXX_2005_03_23_16_33_W_AI__

#include <algorithm>
#include <map>
#include "TImageSegmentation.h"


namespace segmentation {

template <class INDEX_TYPE, INDEX_TYPE idxUndefinedSegment>
inline
INDEX_TYPE * TImageSegmentation<INDEX_TYPE, idxUndefinedSegment>::operator * ()
{
	AssertNonEmpty();

	return &mIndexArray.front();
}

template <class INDEX_TYPE, INDEX_TYPE idxUndefinedSegment>
inline
const INDEX_TYPE * TImageSegmentation<INDEX_TYPE, idxUndefinedSegment>::operator * () const
{
	AssertNonEmpty();

	return &mIndexArray.front();
}

template <class INDEX_TYPE, INDEX_TYPE idxUndefinedSegment>
inline
INDEX_TYPE & TImageSegmentation<INDEX_TYPE, idxUndefinedSegment>::operator () (const long x, const long y)
{
	AssertNonEmpty();

	assert(x>=0 && x<mWidth && y>=0 && y<mHeight);

	return mIndexArray[y*mWidth+x];
}

template <class INDEX_TYPE, INDEX_TYPE idxUndefinedSegment>
inline
const INDEX_TYPE & TImageSegmentation<INDEX_TYPE, idxUndefinedSegment>::operator () (const long x, const long y) const
{
	AssertNonEmpty();

	assert(x>=0 && x<mWidth && y>=0 && y<mHeight);

	return mIndexArray[y*mWidth+x];
}


template <class INDEX_TYPE, INDEX_TYPE idxUndefinedSegment>
TImageSegmentation<INDEX_TYPE, idxUndefinedSegment>::TImageSegmentation(long width, long height)
: mWidth(0), mHeight(0)
{
	if (width>0 && height>0)
		Init(width,height);
}

template <class INDEX_TYPE, INDEX_TYPE idxUndefinedSegment>
TImageSegmentation<INDEX_TYPE, idxUndefinedSegment>::TImageSegmentation(const TImageSegmentation & copy)
{
}

template <class INDEX_TYPE, INDEX_TYPE idxUndefinedSegment>
TImageSegmentation<INDEX_TYPE, idxUndefinedSegment>::~TImageSegmentation()
{
  if (!mIndexArray.empty()) {
    mIndexArray.clear();
  }
	AssertValid();
}

template <class INDEX_TYPE, INDEX_TYPE idxUndefinedSegment>
bool TImageSegmentation<INDEX_TYPE, idxUndefinedSegment>::Init(long width, long height)
{
	AssertValid();

	if (width<=0 || height<=0)
	{
		assert(1);
		return false;
	}

	mWidth = width;
	mHeight = height;

	mIndexArray.resize(width*height);

	// this is done in the "nice" way ... because optimizations like memset are not required here
	std::fill(mIndexArray.begin(),mIndexArray.end(),0);

	AssertValid();

	return true;
}


template <class INDEX_TYPE, INDEX_TYPE idxUndefinedSegment>
bool TImageSegmentation<INDEX_TYPE, idxUndefinedSegment>::GetUsedSegmentIndexes(std::map<INDEX_TYPE, size_t> &indexSet) const
{
	if (!InitDone())
		return false;

	indexSet.clear();

	for (typename INDEX_ARRAY::const_iterator it=mIndexArray.begin(), en=mIndexArray.end(); it!=en; it++)
		indexSet[*it]++; // we assume the default value for the size_t is 0

	return true;
}

template <class INDEX_TYPE, INDEX_TYPE idxUndefinedSegment>
long TImageSegmentation<INDEX_TYPE, idxUndefinedSegment>::GetSegmentCount() const
{
	std::map<INDEX_TYPE, size_t> indexSet;
	if (!GetUsedSegmentIndexes(indexSet))
		return 0;

	return (long)indexSet.size();
}

template <class INDEX_TYPE, INDEX_TYPE idxUndefinedSegment>
bool TImageSegmentation<INDEX_TYPE, idxUndefinedSegment>::ReorderSegmentIndexes()
{
	std::map<INDEX_TYPE, size_t> idxSet;

	typename std::map<INDEX_TYPE, size_t>::iterator setIt,setEn;

	if (!GetUsedSegmentIndexes(idxSet))
		return false;

	std::map<INDEX_TYPE,INDEX_TYPE> idxMap;

	// "undefined" segment indexes will not be affected
	if ((setIt=idxSet.find(idxUndefinedSegment))!=idxSet.end())
	{
		idxMap[idxUndefinedSegment] = idxUndefinedSegment;
		idxSet.erase(setIt);
	}

	// create old-new index mapping
	INDEX_TYPE newIdx=0; 
	for (setIt=idxSet.begin(), setEn=idxSet.end(); setIt!=setEn; setIt++, newIdx++)
		idxMap[(*setIt).first]=newIdx;


	// update segment indexes
	for (typename INDEX_ARRAY::iterator it=mIndexArray.begin(), en=mIndexArray.end(); it!=en; it++)
	{

#if defined(DEBUG) || defined(_DEBUG)
		assert(idxMap.find(*it)!=idxMap.end()); // make sure the lookup map contains all the values
#endif

		*it = idxMap[*it];
	}
	
	return true;
}

} // namespace segmentation

#endif

