#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "motionProfile.h"
#include "ncutImage.h"
#include "CImageSegmentationIndexFile.h"
#include "ncutCore.h"
#include "ncutVis.h"

// #include <arerror.h>
// #include <arbsmat.h>
// #include <arbgsym.h>
// #include <png.h>

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <dirent.h>
#include <regex.h>

#include "stlini.h"
#include "stlini.cpp"

using namespace std;

static regex_t regexpr;
int search (const struct dirent* dp) {
  if (regexec(&regexpr, dp->d_name, 0, NULL, 0) == 0)
    return 1;
  else
    return 0;
}

int readIniFile(const char* inifile); //Declaration

int main(int argc, char *argv[])
{
  if ((argv[1] && !strcmp(argv[1],"--help")) || argc > 3) {
    std::cerr << "USAGE: " << argv[0] << " output_directory" << " [ini file]" << std::endl << std::flush;
    exit(1);
  }
  const char* outputdir = ( argv[1] == NULL) ? "/tmp/":argv[1];
  const char* inifile = ( argv[2] == NULL) ? "":argv[2];

  clock_t totaltimerStart, totaltimerEnd;
  totaltimerStart = clock();
  readIniFile(inifile);
//   

  struct dirent **namelist;
  int n;
//   
  regcomp(&regexpr,".png$", REG_EXTENDED);
  
  if ((n = scandir(".", &namelist, search, alphasort)) < 0) {
    std::cerr << "could not read directory" << std::endl;
    exit(-1);
  };
  
  unsigned long startSegNum = 0;
  ncut::PNGSequence *resultSeq = new ncut::PNGSequence();
  ncut::Segmentation<PNG_IMAGE_DATATYPE, BSI_IMAGE_DATATYPE> seg(3, ncut_Setting);
  ncut::PNGVis<PNG_IMAGE_DATATYPE,BSI_IMAGE_DATATYPE> visualization(&seg);

  clock_t timerStart, timerEnd;
  clock_t timerStartFrame, timerEndFrame;
  for (int i = 0; i < n; i++) {
    timerStartFrame = clock();
    try {
#ifdef debugout
      std::cout << std::endl << std::endl << "loading the images (" << namelist[i]->d_name << ") into memory...\n" << std::flush;
#endif
      
      ncut::PNGImage png(namelist[i]->d_name);
#ifdef debugout
      std::cout << "width " << png.width() << std::endl << std::flush;
      std::cout << "heigh " << png.height() << std::endl << std::flush;
#endif
      
    // cut of the ".png" of the name
      char name[strlen(namelist[i]->d_name) - 3];
      strncpy(name, namelist[i]->d_name, strlen(namelist[i]->d_name) - 3);
      name[strlen(namelist[i]->d_name) - 4] = '\0';

      ncut::BSIImage bsi(strncat(name,".bsi",8), startSegNum);
#ifdef debugout
      std::cout << "width " << bsi.width() << std::endl << std::flush;
      std::cout << "heigh " << bsi.height() << std::endl << std::flush;
      std::cout << "nSeg " << bsi.nSeg() << std::endl << std::flush;

      std::cout << startSegNum << std::endl << std::flush;

      timerStart = clock();
#endif

      if (seg.push(&png,&bsi) == 1)  // There are not enough images in the sequence for segmentation
        continue;
#ifdef debugout
      timerEnd = clock();

      std::cout << "done Ncut (" << ((double)(timerEnd - timerStart))/CLOCKS_PER_SEC << " seconds)" << std::endl << std::flush;
#endif
      
      resultSeq->clear();
      visualization.create(*resultSeq, i);
#ifdef debugout
      std::cout << "saving the visualization to disk..." << std::flush;
      
      timerStart = clock();
#endif


      std::stringstream currentFilename;
      
      if (i < 10) {
        currentFilename << outputdir << "result000" << i << ".png";
      }
      else if (i < 100) {
        currentFilename << outputdir << "result00" << i << ".png";
      }
      else if (i < 1000) {
        currentFilename << outputdir << "result0" << i << ".png";
      }      
      else {
        currentFilename << outputdir << "result" << i << ".png";
      }
#ifdef debugout
      std::cout << currentFilename.str().c_str() << std::endl << std::flush;
#endif

      int saveReturn = resultSeq->save(currentFilename.str().c_str());
#ifdef debugout
      std::cout << "sequence save returned " << saveReturn << std::endl << std::flush;

      timerEnd = clock();
      std::cout << "done Saving (" << ((double)(timerEnd - timerStart))/CLOCKS_PER_SEC << " seconds)" << std::endl << std::flush;    
#endif

    }
    catch (ncut::ncutCritical& e) {
      std::cout << "critical exception : " << e.what() << std::endl <<
          std::flush;
      delete resultSeq;
      return -1;
    }
    catch (ncut::ncutException& e) {
      std::cout << "exception : " << e.what() << std::endl << std::flush;
    }
#ifdef debugout
    timerEndFrame = clock();
    std::cout << "done complete frame (" << ((double)(timerEndFrame - timerStartFrame))/CLOCKS_PER_SEC << " seconds)" << std::endl << std::flush;
#endif

  }
 
  delete resultSeq;
	
	totaltimerEnd = clock();
	std::cout << "total time: " << ((double)(totaltimerEnd - totaltimerStart))/CLOCKS_PER_SEC << " seconds" << std::endl << std::flush;
	
  
  std::cout << "saving coordinates .... " << std::flush;
  
  std::map<BSI_IMAGE_DATATYPE, std::vector<int> >::const_iterator remPosX;
  std::map<BSI_IMAGE_DATATYPE, std::vector<int> >::const_iterator remPosY;
  
  for (remPosX=visualization.rememberedPosX().begin(),
       remPosY=visualization.rememberedPosY().begin();
       remPosX != visualization.rememberedPosX().end(); remPosX++, remPosY++) {

         std::vector<int>::const_iterator locationX;
         std::vector<int>::const_iterator locationY;

         // if this segment existed for only a few frames, ignore it
         if (remPosX->second.size() < ncut_Setting.segment_adult_age) continue;

         std::stringstream filename;
         filename << "/tmp/coords_" << remPosX->first << ".txt";
         FILE *file = fopen(filename.str().c_str(),"w");
         fprintf(file,": %d\n", visualization.firstFrame(remPosX->first));
         std::map<BSI_IMAGE_DATATYPE,double>::const_iterator segIt = visualization.segmentSumArea().find(remPosX->first);
         double area;
         if (segIt != visualization.segmentSumArea().end())
           area = segIt->second;
         else
           area = 0;
             
         fprintf(file,":: %f\n", (area/(double)remPosX->second.size())/3 );
         for (locationX = remPosX->second.begin(),
              locationY = remPosY->second.begin();
              locationX != remPosX->second.end(); locationX++,locationY++) {
                fprintf(file,"%d %d\n",*locationX,*locationY);
              }
         fclose(file);
  }
  std::cout << "done" << std::endl << std::flush;

  return EXIT_SUCCESS;
}

int readIniFile(const char* inifile) {
  INIFile ini = LoadIni(inifile);

  if (!ini.size()) {
    std::cout << "ini file does not exist ... using default values" << std::endl << std::flush;
  }
  else {
    std::cout << "found ini file ... loading values" << std::endl << std::flush;
    if (GetIniSetting(ini, "ncut","ncut_tree_depth").compare("") != 0) {
      ncut_Setting.ncut_tree_depth = atoi(GetIniSetting(ini,
"ncut","ncut_tree_depth").c_str());
    }
    else {
      std::cout << "ncut_tree_depth not found in ncut.ini ... using default value" << std::endl << std::flush;
    }

    if (GetIniSetting(ini, "ncut","number_of_eigenvectors").compare("") != 0) {
      ncut_Setting.number_of_eigenvectors = atoi(GetIniSetting(ini, "ncut","number_of_eigenvectors").c_str());
    }
    else {
      std::cout << "number_of_eigenvectors not found in ncut.ini ... using default value" << std::endl << std::flush;
    }

    
    if (GetIniSetting(ini, "ncut","min_segment_attach_similarity").compare("")
!= 0 ) {
      ncut_Setting.min_segment_attach_similarity = atof(GetIniSetting(ini,
"ncut","min_segment_attach_similarity").c_str());
    }
    else {
      std::cout << "min_segment_attach_similarity not found in ncut.ini ..." <<
      "using default value" << std::endl << std::flush;
    }

    if (GetIniSetting(ini, "ncut","visualisation_mode").compare("") != 0) {
			ncut_Setting.visualisation_mode = atoi(GetIniSetting(ini, "ncut","visualisation_mode").c_str());
    }
    else {
      std::cout << "visualisation_mode not found in ncut.ini ... using default value" << std::endl << std::flush;
    }

    if (GetIniSetting(ini,
"ncut","number_of_eigenvector_splittest_steps").compare("") != 0) {
      ncut_Setting.number_of_eigenvector_splittest_steps =
atoi(GetIniSetting(ini,
"ncut","number_of_eigenvector_splittest_steps").c_str());
    }
    else {
      std::cout << "splittestSteps not found in ncut.ini ... using default value" << std::endl << std::flush;
    }

    if (GetIniSetting(ini, "ncut","drawTrace").compare("") != 0) {
      ncut_Setting.drawTrace = atoi(GetIniSetting(ini,
"ncut","drawTrace").c_str());
    }
    else {
      std::cout << "drawTrace not found in ncut.ini ... using default value" << std::endl << std::flush;
    }

    if (GetIniSetting(ini, "ncut","drawCross").compare("") != 0) {
      ncut_Setting.drawCross = atoi(GetIniSetting(ini, "ncut","drawCross").c_str());
    }
    else {
      std::cout << "drawCross not found in ncut.ini ... using default value" << std::endl << std::flush;
    }

    if (GetIniSetting(ini, "ncut","drawLine").compare("") != 0) {
      ncut_Setting.drawLine = atoi(GetIniSetting(ini, "ncut","drawLine").c_str());
    }
    else {
      std::cout << "drawLine not found in ncut.ini ... using default value" << std::endl << std::flush;
    }

    if (GetIniSetting(ini, "ncut","tracelength").compare("") != 0) {
      ncut_Setting.tracelength = atoi(GetIniSetting(ini, "ncut","tracelength").c_str());
    }
    else {
      std::cout << "tracelength not found in ncut.ini ... using default value" << std::endl << std::flush;
    }

    if (GetIniSetting(ini, "ncut","merge_segments").compare("") != 0) {
      ncut_Setting.merge_segments = atoi(GetIniSetting(ini,
"ncut","merge_segments").c_str());
    }
    else {
      std::cout <<
          "merge_segments not found in ncut.ini ... using default value" <<
std::endl << std::flush;
    }

    if (GetIniSetting(ini, "ncut","connected_segments").compare("") != 0) {
      ncut_Setting.connected_segments = atoi(GetIniSetting(ini,
"ncut","connected_segments").c_str());
    }
    else {
      std::cout <<
          "connected_segments not found in ncut.ini ... using default value" <<
std::endl << std::flush;
    }

    if (GetIniSetting(ini, "ncut","segment_delay").compare("") != 0) {
      ncut_Setting.segment_delay = atoi(GetIniSetting(ini,
"ncut","segment_delay").c_str());
    }
    else {
      std::cout << "segment_delay not found in ncut.ini ... using default value"
<< std::endl << std::flush;
    }

    if (GetIniSetting(ini, "ncut","stretch_saturation").compare("") != 0) {
      ncut_Setting.stretch_saturation = atoi(GetIniSetting(ini,
"ncut","stretch_saturation").c_str());
    }
    else {
      std::cout <<
      "stretch_saturation not found in ncut.ini ... using default value" <<
std::endl << std::flush;
    }

    if (GetIniSetting(ini, "ncut","stretch_saturation_merge").compare("") != 0)
{
  ncut_Setting.stretch_saturation_merge = atoi(GetIniSetting(ini,
"ncut","stretch_saturation_merge").c_str());
    }
    else {
      std::cout <<
      "stretch_saturation_merge not found in ncut.ini ... using default value"
      << std::endl << std::flush;
    }

    if (GetIniSetting(ini, "ncut","merge_threshold").compare("") != 0) {
      ncut_Setting.merge_threshold = atoi(GetIniSetting(ini,
"ncut","merge_threshold").c_str());
    }
    else {
      std::cout <<
          "merge_threshold not found in ncut.ini ... using default value" <<
std::endl << std::flush;
    }
  }
  std::cout << "depth:" << ncut_Setting.ncut_tree_depth << std::endl << std::flush;
  std::cout << "number_of_eigenvectors:" << ncut_Setting.number_of_eigenvectors
<< std::endl << std::flush;
  std::cout << "maxsim:" << ncut_Setting.min_segment_attach_similarity <<
std::endl << std::flush;
  std::cout << "splittestSteps:" << ncut_Setting.number_of_eigenvector_splittest_steps << std::endl << std::flush;
  std::cout << "drawTrace:" << ncut_Setting.drawTrace << std::endl << std::flush;
  std::cout << "drawCross:" << ncut_Setting.drawCross << std::endl << std::flush;
  std::cout << "drawLine:" << ncut_Setting.drawLine << std::endl << std::flush;
  std::cout << "tracelength:" << ncut_Setting.tracelength << std::endl << std::flush;
  std::cout << "visualisation_mode:" << ncut_Setting.visualisation_mode << std::endl << std::flush;
  std::cout << "mergeSegments:" << ncut_Setting.merge_segments << std::endl << std::flush;
  std::cout << "connectedSegments:" << ncut_Setting.connected_segments << std::endl << std::flush;
  std::cout << "segmentDelay:" << ncut_Setting.segment_delay << std::endl << std::flush;
  std::cout << "stretchSaturation:" << ncut_Setting.stretch_saturation << std::endl << std::flush;
  std::cout << "stretchSaturationMerge:" << ncut_Setting.stretch_saturation_merge << std::endl << std::flush;
  std::cout << "mergeThreshold:" << ncut_Setting.merge_threshold << std::endl << std::flush;

  return 0;
}
